/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.app.view.report.NewReportEditor");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.app.internal.common.ImagePool");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Tree");
dojo.require("dojo.data.ItemFileReadStore");

dojo.requireLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");

// Global variable to store the controller. Required to make
// the controller available to the code generated by the REST
// service.
reportController = null;

(function() { // module container

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var session = com.ibm.team.repository.web.client.session;
var ImagePool = com.ibm.team.reports.web.app.internal.common.ImagePool;

var SHARED = 1 << 0;
var PRIVATE = 1 << 2;
var SYSTEM = 1 << 3;

/**
 * Widget to create a new report query
 */
dojo.declare("com.ibm.team.reports.web.app.view.report.NewReportEditor", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "app/view/report/templates/NewReportEditor.html"),
		//templateCssPath: dojo.moduleUrl("com.ibm.team.reports.web", "app/view/report/templates/NewReportEditor.css"),
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");
			this.reportTitle = this.messages.reportTitle;
			this.reportName = this.messages.name;
			this.description = this.messages.description;
			this.folder = this.messages.folder;
			this.template = this.messages.template;
			this.save = this.messages.save;
		},
		
		postCreate: function() {
			this._myFolders = null;
			this._sharedFolders = null;
			this._allFolders = [];
			this._foundMyFolders = false;
			this._foundSharedFolders = false;
			
			this.typeIcon.src = ImagePool.REPORT_OBJ;
			
			var responseHandler = new ServiceResponseHandler(
					this,
					"_addTemplates",
					"_remoteError");
			ReportsClient.getReportDescriptors(responseHandler, {});
			
			var args = {
				ownerUUID: session.getAuthenticatedContributor().itemId,
				flags: PRIVATE
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addMyFolders",
					"_remoteError");
			ReportsClient.getFolderDescriptors(responseHandler, args);
			args = {
				flags: (SHARED | SYSTEM)
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addSharedFolders",
					"_remoteError");
			ReportsClient.getFolderDescriptors(responseHandler, args);
		},
		
		_handleSave: function() {
			var args = {
		        name: this._getName(),
		        description: this._getDescription(),
		        reportUUID: this._currentTemplate.reportUUID
	        }
	        // If this is a special folder, just set the sharing flag. Otherwise
	        // set the parent folder.
	        if (this._currentFolder.teamAreaUUID == null) {
	        	args.sharedIfNullFolder = this._currentFolder.shared;
	        } else {
	        	args.folderUUID = this._currentFolder.folderUUID;
	        }
	        var responseHandler = new ServiceResponseHandler(
					this,
					"_saveComplete",
					"_remoteError");
			ReportsClient.postSaveQueryDescriptor(responseHandler, args);
		},
		
		_getName: function() {
			return this._nameInput.value;
		},
		
		_getDescription: function() {
			return this._descriptionText.value;
		},
		
		_saveComplete: function(args) {
			this.controller.refreshNavigator();
			document.location.href = "#action=com.ibm.team.reports.viewQuery&queryUUID=" + args;
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._saveButton, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			alert(dojo.string.substitute(this.messages.errorWithMessage, [error.message]));
		},

		_addTemplates: function(args) {
			if (args == null) args = [];
			// First sort them alphabetically
			function sortFunction(a, b) {
				return a.name < b.name ? -1 : (a.name > b.name ? 1 : 0);
			}
			var sorted = args.sort(sortFunction);
			this._templates = sorted;
			var store = this._createTemplateDataStore(args);
			this._createTemplateTree(store);			
		},
		
		_addSharedFolders: function(args) {
			if (args == null) args = [];
			this._sharedFolders = args;
			for (var i = 0; i < args.length; i++) {
				this._allFolders[this._allFolders.length] = args[i];
			}
			this._foundSharedFolders = true;
			if (this._foundMyFolders) {
				this._createFolderTree();
			}
		},

		_addMyFolders: function(args) {
			if (args == null) args = [];
			this._myFolders = args;
			for (var i = 0; i < args.length; i++) {
				this._allFolders[this._allFolders.length] = args[i];
			}
			this._foundMyFolders = true;
			if (this._foundSharedFolders) {
				this._createFolderTree();
			}
		},

		_createFolderDataStore: function() {
			var store = new Object();
			store.label = "name";
			store.identifier = "folderUUID";
			var rootItems = new Array();
			
			var sharedRoot = null;
			var privateRoot = null;
			// A map of all folders from UUID -> DTO
			var folderMap = {};
			
			// First, find the root folders and set up folderMap
			for (var i = 0; i < this._allFolders.length; i++) {
				var folder = this._allFolders[i];
				if (folder.system) {
					rootItems[rootItems.length] = folder;
					if (folder.shared) {
						sharedRoot = folder;
					} else {
						privateRoot = folder;
					}
				}
				if (folder.folderUUID != null) {
					folderMap[folder.folderUUID] = folder; 
				}
			}
			for (var i = 0; i < this._allFolders.length; i++) {
				var folder = this._allFolders[i];
                if (!folder.system) {
                    var parentUUID = folder.parentUUID;
                    var parent = null;
                    if (parentUUID != null) {
                    	parent = folderMap[parentUUID];
                    }
                    if (parent == null) {
                        if ((!folder.shared)
                                && (privateRoot != null)
                                && (folder.name != 
                                        privateRoot.name)) {
                            parent = privateRoot;
                        } else if ((folder.shared)
	                            && (sharedRoot != null)
	                            && (folder.name != sharedRoot.name)) {
	                        parent = sharedRoot;
                        }
                    }
                    if (parent != null) {
                    	var list = parent.children;
                        if (list == null) {
                        	list = [];
                        	parent.children = list;
                        }
                        list[list.length] = folder;
                    }
                }
            }
			
			store.items = rootItems;
			return store;
		},
		
		_createTemplateDataStore: function(templates) {
			var store = new Object();
			store.label = "name";
			store.identifier = "reportUUID";
			var items = new Array();
			store.items = templates;
			return store;
		},
		
		_createFolderTree: function() {
			var dataStore = this._createFolderDataStore();
			var treeStore = new dojo.data.ItemFileReadStore({
				data: dataStore
			});
			var tree = new dijit.Tree({
				store: treeStore,
				widgetId: "folderTreeWidget",
				getIconClass: function(item) {
					return "iconFolder";
				}
			});

			this._folderTreeWidget = tree;
			this._folderTree.appendChild(tree.domNode);
			
			this.connect(tree, "focusNode", this._onFocusFolder);
		},

		_createTemplateTree: function(dataStore) {
			var treeStore = new dojo.data.ItemFileReadStore({
				data: dataStore
			});
			var tree = new dijit.Tree({
				store: treeStore,
				widgetId: "templateTreeWidget",
				getIconClass: function(item) {
					return "iconTemplate";
				}
			});

			this._templateTreeWidget = tree;
			this._templateTree.appendChild(tree.domNode);
			
			this.connect(tree, "focusNode", this._onFocusTemplate);
		},

		_onFocusFolder: function(node) {
			// Get the old focus node
			var oldFocusNode = this._currentFolderNode;
		
			// Remove selection from old focus node
			if ((node != oldFocusNode) &&
				(oldFocusNode != null)){
				dojo.removeClass(oldFocusNode.contentNode, "tree-focus-entry");
			}
			// Add selection to new focus node
			if (node != null) {
				dojo.addClass(node.contentNode, "tree-focus-entry");
			}
						
			if (node) {
				this._currentFolder = node.item;
				this._currentFolderNode = node;
			} else {
				this._currentFolder = null;
				this._currentFolderNode = null;
			}
			
			this._updateEnablement();
		},
				
		_onFocusTemplate: function(node) {
			// Get the old focus node
			var oldFocusNode = this._currentTemplateNode;
		
			// Remove selection from old focus node
			if ((node != oldFocusNode) &&
				(oldFocusNode != null)){
				dojo.removeClass(oldFocusNode.contentNode, "tree-focus-entry");
			}
			// Add selection to new focus node
			if (node != null) {
				dojo.addClass(node.contentNode, "tree-focus-entry");
			}
			
			if (node) {
				this._currentTemplate = node.item;
				this._currentTemplateNode = node;
			} else {
				this._currentTemplate = null;
				this._currentTemplateNode = null;
			}
			
			this._updateEnablement();
		},
				
		_handleNameChange: function() {
			this._updateEnablement();
		},
		
		_updateEnablement: function() {
			var name = this._getName();
			var hasName = (name && name.length > 0);
			this._saveButton.disabled = !(hasName && this._currentTemplate && this._currentFolder);
		}
	}
);

})();