/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.client.internal.PlanningClient"); //$NON-NLS-1$

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.internal.mixins.ContributorAbsence"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.mixins.Iteration"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.mixins.IterationPlan"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.client.internal.mixins.PlanItem"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$


(function() { // module container

var ServiceRequest 			= com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;

var SERVICE_URI				= "com.ibm.team.apt.internal.service.rest.IPlanRestService"; //$NON-NLS-1$

var Assert					= com.ibm.team.apt.web.ui.internal.common.Assert;
var ReferencedItemStore		= com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore;
var ITEM_TYPES				= com.ibm.team.apt.web.ui.internal.common.ReferencedItemStore.ITEM_TYPES;


com.ibm.team.apt.web.client.internal.PlanningClient= {

	// ---- API ------------------------------------------------------------------------------------------------------------	
	
	/**
	 * 
	 * @param {Object} responseHandler
	 * @param {Object} itemType
	 * @param {Array, String} itemIds
	 */
	fetchUIItems: function(responseHandler, itemType, itemIds) {
		var serviceArguments= {};
		serviceArguments["itemType"]= itemType; //$NON-NLS-1$
		serviceArguments["itemIds"]= dojo.isArray() ? itemIds : [itemIds]; //$NON-NLS-1$
		
		this._invokePlanService(responseHandler, "getUIItems", serviceArguments); //$NON-NLS-1$
	},

	searchIterationPlans: function(responseHandler, queryDescriptor, startIndex, maxResults, searchToken) {
		var serviceArguments= {};
		
		serviceArguments["keywords"]= queryDescriptor.getKeywords() || ""; //$NON-NLS-1$ //$NON-NLS-2$
		serviceArguments["relationshipType"]= queryDescriptor.getRelationshipType() || ""; //$NON-NLS-1$ //$NON-NLS-2$
		serviceArguments["relatedItemId"]= queryDescriptor.getRelatedItemId() || ""; //$NON-NLS-1$ //$NON-NLS-2$
		serviceArguments["currentOnly"]= queryDescriptor.isCurrentOnly(); //$NON-NLS-1$
		serviceArguments["startIndex"]= startIndex; //$NON-NLS-1$
		serviceArguments["maxResults"]= maxResults; //$NON-NLS-1$
		serviceArguments["token"]= searchToken || ""; //$NON-NLS-1$ //$NON-NLS-2$

		this._invokePlanService(responseHandler, "getIterationPlanSearchResults", serviceArguments); //$NON-NLS-1$
	},

	fetchIterationPlan: function(responseHandler, planId) {
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getIterationPlan", serviceArguments); //$NON-NLS-1$
	},

	fetchAttachedPage: function(responseHandler, planId, pageId) {
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$
		serviceArguments["pageId"]= pageId; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getAttachedPage", serviceArguments); //$NON-NLS-1$
	},

	fetchPlannedWorkItems: function(responseHandler, planId) {
		var serviceArguments= {};
		serviceArguments["planId"]= planId; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getPlannedWorkItems", serviceArguments); //$NON-NLS-1$
	},

	fetchTeamLoad: function(responseHandler, planIds) {
		var serviceArguments= {};
		serviceArguments["planIds"]= planIds; //$NON-NLS-1$

		this._invokePlanService(responseHandler, "getTeamLoad", serviceArguments); //$NON-NLS-1$
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	/**
	 * Invokes the requested method on the APT REST service.
	 */
	_invokePlanService: function(responseHandler, method, args) {
		var _self= this;
		var dtoMassageInterceptor= new ServiceResponseHandler({
			_success: function(result) {
				responseHandler.doCallback(_doMassageDTO(result));
			},
			_error: function(errorObj) {
				responseHandler.doErrorCallback(errorObj);
			}
		}, "_success", "_error"); //$NON-NLS-1$ //$NON-NLS-2$

		
		this._handleProgressMonitor(responseHandler);
		
		var currentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		var serviceArguments= dojo.mixin(args || {}, { projectAreaId: currentProjectArea.itemId });
		var serviceRequest= new com.ibm.team.repository.web.transport.ServiceRequest(SERVICE_URI, method, serviceArguments);

		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(serviceRequest, dtoMassageInterceptor);
	},
	
	_handleProgressMonitor: function(responseHandler) {
		if (dojo.isFunction(responseHandler["doStart"])) { //$NON-NLS-1$
			responseHandler.doStart();
		}
	},

	__sentinel: null // terminates this class definition
};

// ---- DTO massage --------------------------------------------------------------------------------------------------------

var CLASSNAME_PROPERTY= "_eQualifiedClassName"; //$NON-NLS-1$
var CLASSNAME_EXPRESSION= /com\.ibm\.team\.apt\.rest\.dto:([a-zA-Z0-9_]*)/

var MassageInfo= {
	"IterationItemDTO": { //$NON-NLS-1$
		mixin: com.ibm.team.apt.web.client.internal.mixins.Iteration
	}, 

	"IterationPlanDTO": { //$NON-NLS-1$
		arrays: [ "attachedPages", "contributedPages" ], //$NON-NLS-1$ //$NON-NLS-2$
		items: [ 
			{ name: "teamArea", type: ITEM_TYPES.TEAMAREA }, //$NON-NLS-1$
			{ name: "iteration", type: ITEM_TYPES.ITERATION } //$NON-NLS-1$
		],
		mixin: com.ibm.team.apt.web.client.internal.mixins.IterationPlan
	},
	
	"SearchIterationPlansResultDTO": { //$NON-NLS-1$
		arrays: [ "results", "referencedItems" ] //$NON-NLS-1$ //$NON-NLS-2$
	}, 
	
	"IterationPlanTeamLoadResultDTO": { //$NON-NLS-1$
		arrays: [ "loadInfos" ] //$NON-NLS-1$
	},
	
	"PlannedItemsPageDTO": { //$NON-NLS-1$
		arrays: [ "myFolders", "workItems", "referencedItems", "teamMemberWorkLoad" ] //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
	}, 

	"MyFolderDTO": { //$NON-NLS-1$
		arrays: [ "subFolders" ] //$NON-NLS-1$
	}, 
	
	"WorkItemDTO": { //$NON-NLS-1$
		arrays: [ "mySortOrder", "children" ], //$NON-NLS-1$ //$NON-NLS-2$
		items: [
			{ name: "workItemType", type: ITEM_TYPES.WITYPE }, //$NON-NLS-1$
			{ name: "priority", type: ITEM_TYPES.PRIORITY }, //$NON-NLS-1$
			{ name: "severity", type: ITEM_TYPES.SEVERITY }, //$NON-NLS-1$
			{ name: "owner", type: ITEM_TYPES.CONTRIBUTOR }, //$NON-NLS-1$
			{ name: "status", type: ITEM_TYPES.WISTATE }, //$NON-NLS-1$
			{ name: "category", type: ITEM_TYPES.CATEGORY }, //$NON-NLS-1$
			{ name: "iteration", type: ITEM_TYPES.ITERATION } //$NON-NLS-1$
		],
		mixin: com.ibm.team.apt.web.client.internal.mixins.PlanItem
	},
	
	"ContributorWorkLoadDTO": { //$NON-NLS-1$
		arrays: [ "absences" ] //$NON-NLS-1$
	},

	"ContibutorAbsenceDTO": { //$NON-NLS-1$
		mixin: com.ibm.team.apt.web.client.internal.mixins.ContributorAbsence
	}
};

function _doMassageDTO(dto) {
	dto._referencedItemStore= null;
	
	switch (_extractClassName(dto)) {
		case "LoadIterationPlanResultDTO": //$NON-NLS-1$
		case "SearchIterationPlansResultDTO": //$NON-NLS-1$
		case "PlannedItemsPageDTO": //$NON-NLS-1$
			dto._referencedItemStore= new ReferencedItemStore(dto["referencedItems"]) //$NON-NLS-1$
			break;
	}
	
	_applyMassage(dto, dto._referencedItemStore);
	
	return dto;
};

function _applyMassage(dto, referencedItemStore) {
	var className= _extractClassName(dto);
	if (className) {
		var massageInfo= MassageInfo[className];
		if (massageInfo) {
			// 1) ensure all arrays are created, so that we do not have to clutter the code later on with tons of ifs
			var arrays= massageInfo.arrays || [];
			for (var i = 0; i < arrays .length; i++) {
				dto[arrays[i]]= dto[arrays[i]] || [];
			}
			
			// 2) add item accessors
			var items= massageInfo.items || [];
			Assert.isTrue(items.length == 0 || referencedItemStore != null, "Missing ReferencedItemStore"); //$NON-NLS-1$
			for (var i = 0; i < items.length; i++) {
				_addItemAccessor(dto, items[i].name, items[i].type, referencedItemStore);
			}
			
			// 3) mixin additions
			if (massageInfo.mixin) {
				dojo.mixin(dto, massageInfo.mixin);
			}
		}
		
		for (var property in dto) {
			if (dto[property] != null && dojo.isObject(dto[property])) {
				_applyMassage(dto[property], referencedItemStore);
			}
		}
	} else if (dojo.isArrayLike(dto)) {
		for (var i= 0; i < dto.length; i++) {
			if (dto[i] != null && dojo.isObject(dto[i])) {
				_applyMassage(dto[i], referencedItemStore);
			}
		}
	}
};

function _extractClassName(dto) {
	var result= CLASSNAME_EXPRESSION.exec(dto[CLASSNAME_PROPERTY]);
	return result ? result[1] : null;
};

function _addItemAccessor(dto, property, type, referencedItemStore) {
	var accessorName= "get" + property.substr(0, 1).toUpperCase() + property.substr(1); //$NON-NLS-1$
	dto[accessorName]= function() {
		return referencedItemStore.getItem(type, dto[property]);
	};
};


})();