/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.client.internal.TeamsClient");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

(function() {

/*
 * Aliases
 */
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;

/*
 * Constants
 */
var _SERVICE_URI = "com.ibm.team.dashboard.viewlets.service.internal.teams.ITeamsService";

/*
 * Retrieves the teams of the specified project area.
 */
com.ibm.team.dashboard.viewlets.web.client.internal.TeamsClient.getTeamsForProjectArea =
/* ProcessAreaDTO[] */ function(/* string */ itemId, /* boolean */ scopeSubteams, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
	var params = {itemId: itemId};
	if (scopeSubteams)
		params.scopeSubteams = true;
	var req = new ServiceRequest(_SERVICE_URI, "getTeamsForProjectArea", params);
	if (tracker) {
		tracker.register(req, srh);
	}
	TeamServerClient.invokeService(req, srh);
}

/*
 * Retrieves the sub-teams of the specified team area.
 */
com.ibm.team.dashboard.viewlets.web.client.internal.TeamsClient.getTeamsForTeamArea =
/* ProcessAreaDTO[] */ function(/* string */ itemId, /* boolean */ scopeSubteams, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
	var params = {itemId: itemId};
	if (scopeSubteams)
		params.scopeSubteams = true;
	var req = new ServiceRequest(_SERVICE_URI, "getTeamsForTeamArea", params);
	if (tracker) {
		tracker.register(req, srh);
	}
	TeamServerClient.invokeService(req, srh);
}

/*
 * Private Method
 * 
 * Retrieves the project area name
 * 
 * @param String itemId
 * @param String itemScope
 * @param ServiceResponseHandler srh
 * @return <none>
 */	
com.ibm.team.dashboard.viewlets.web.client.internal.TeamsClient.getProjectAreaName =
function(itemId, itemScope, handler, /* ServiceTracker */ tracker) {
	var request = new ServiceRequest(_SERVICE_URI, "getProjectAreaName", {itemId: itemId, itemScope: itemScope});
	if (tracker) {
		tracker.register(request, handler);
	}
	TeamServerClient.invokeService(request, handler);
}

})();