/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.client.internal.ViewletRegistry");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

(function() {

//
// Aliases
//
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

/*
 * A client-side data caching layer for the viewlet registry.
 */
com.ibm.team.dashboard.web.client.internal.ViewletRegistry = {
	
	_definitionsById: new Object(),
	_hasDefinitions: false,
	_hierarchiesByScopeItemId: new Object(),
	
	/*
	 * Adds the given definitions to the registry. This is needed for
	 * piggy-backing definitions in other requests, e.g. dashboard request.
	 */
	addDefinitions: function(/* ViewletDefinitionDTO[] */ viewletDefinitionDTOs) {
		for (var i=0;i<viewletDefinitionDTOs.length;i++) {
			var definition = viewletDefinitionDTOs[i];
			this._definitionsById[definition.id] = definition;
		}
		if (viewletDefinitionDTOs.length > 0) {
			this._hasDefinitions = true;
		}
	},
	
	/*
	 * Returns the given viewlet definitions (asynchronously).
	 */
	getDefinitions: function(/* string[] */ definitionIds, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		var foundDefs = new Array();
		var missingDefIds = new Array();
		for (var i=0;i<definitionIds.length;i++) {
			var def = this._definitionsById[definitionIds[i]];
			if (def) {
				foundDefs.push(def);
			}
			else {
				missingDefIds.push(definitionIds[i]);
			}
		}
		if (missingDefIds.length >= 1) {
			// fetch missing ones
			var serviceObj = {
				_foundDefs: foundDefs,
				_srh: srh,
				_viewletRegistry: this,
				success: function(/* ViewletDefinitionDTO[] */ viewletDefinitionDTOs) {
					this._viewletRegistry.addDefinitions(viewletDefinitionDTOs);
					this._srh.doCallback(this._foundDefs.concat(viewletDefinitionDTOs));
				},
				failure: function(/* Error */ error) {
					this._srh.doErrorCallback(error);
				}
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			DashboardClient.getViewletDefinitions(missingDefIds, srh, tracker);
		}
		else {
			// return cached definitions
			srh.doCallback(foundDefs);
		}
	},

	/*
	 * Returns the viewlet definitions for all viewlets in the given dashboard
	 * (asynchronously).
	 */
	getDefinitionsFor: function(/* DashboardItem */ dashboardItem, /* ServiceResponseHandler */ srh, /* ServiceTracker */ tracker) {
		// scan viewlets in the dashboard
		var definitionIdsSet = new Object();
		var viewletPages = dashboardItem.getViewletPages();
		for (var i=0;i<viewletPages.length;i++) {
			var columns = viewletPages[i].getColumns();
			for (var j=0;j<columns.length;j++) {
				var viewlets = columns[j].getViewlets();
				for (var k=0;k<viewlets.length;k++) {
					definitionIdsSet[viewlets[k].getDefinitionId()] = true;
				}
			}
		}
		// transform set to array
		var definitionIds = new Array();
		for (var i in definitionIdsSet) {
			definitionIds.push(i);
		}
		this.getDefinitions(definitionIds, srh, tracker);
	},
	
	/*
	 * Returns the hierarchy of categories and viewlet entries (asynchronously).
	 */	
	getHierarchy: function(/* number */ scope, /* string */ scopeItemId, /* ServiceResponseHandler */ srh) {
		// check cache
		var hierarchy = this._hierarchiesByScopeItemId[scopeItemId];
		if (hierarchy) {
			// return cached hierarchy
			srh.doCallback(hierarchy);
		}
		else {
			// request from server
			var serviceObj = {
				_scopeItemId: scopeItemId,
				_srh: srh,
				_viewletRegistry: this,
				success: function(/* ViewletHierarchyDTO */ viewletHierarchyDTO) {
					// cache and return
					this._viewletRegistry._hierarchiesByScopeItemId[this._scopeItemId] = viewletHierarchyDTO;
					this._srh.doCallback(viewletHierarchyDTO);
				},
				failure: function(/* Error */ error) {
					this._srh.doErrorCallback(error);
				}
			};
			var srh2 = new ServiceResponseHandler(serviceObj, "success", "failure");
			DashboardClient.getViewletHierarchy(scope, scopeItemId, srh2);
		}
	},
	
	/*
	 * Returns whether or not the registry has any viewlet definitions.
	 */
	/* boolean */ hasDefinitions: function() {
		return this._hasDefinitions;
	}
};

})();
