/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.DashboardWelcome");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");

dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var messages;

/*
 * A welcome widget that appears in the dashboard content area, guiding
 * the user through creating their initial dashboard.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.DashboardWelcome",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/DashboardWelcome.html"),
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this._scope = args.scope;
		this._scopeItemId = args.scopeItemId;
		this._isPublic = args._isPublic;
		this._info = args.info;
		this._dashboardPage = args.dashboardPage;
	},
		
	populate: function() {
		if (this._scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
			if (getAuthenticatedContributor().itemId == this._scopeItemId) {
				this._title.innerHTML = dojo.string.substitute(messages.profileTitle, [HTMLUtility.escapeHTML(getAuthenticatedContributor().name)]);
				if (this._info.permissions.create) {
					var nodes = I18nUtil.addMessage(this._welcomeMessage, messages.welcomeUserCreate);
					this.connect(nodes[0], "onclick", this._handleNewDashboard);
				} else {
					this._welcomeMessage.appendChild(document.createTextNode(messages.welcomeUserNoPermission));				
				}
			} else {
				this._title.innerHTML = dojo.string.substitute(messages.profileTitle, [HTMLUtility.escapeHTML(this._info.scopeItemName)]);
				this._welcomeMessage.appendChild(document.createTextNode(dojo.string.substitute(messages.welcomeUserNone, [this._info.scopeItemName])));
			}
		}
		else {
			this._title.innerHTML = HTMLUtility.escapeHTML(this._info.scopeItemName);
			
			if (this._info.permissions.create) {
				var nodes = I18nUtil.addMessage(this._welcomeMessage, messages.welcomeProcessCreate);
				this.connect(nodes[0], "onclick", this._handleNewDashboard);
			}
			else {
				this._welcomeMessage.appendChild(document.createTextNode(messages.welcomeProcessNoPermission));
			}
		}
	},
	
	getScope: function() {
		return this._scope;
	},
	
	getScopeItemId: function() {
		return this._scopeItemId;
	},
	
	isDashboardPublic: function() {
		return this._isPublic;
	},
	
	// use the same method for public personal and process
	// just set isPublic to true always since it only matters for contributor case
	_handleNewDashboard: function(event) {
		event.preventDefault();
		var self = this;
		var layout = this._layout;
		var serviceObj = {
			success: function(/* ResponseDTO */ responseDTO) {
				switch (responseDTO.status) {
					case "OK":
						self._dashboardPage._initializeContentWidget(responseDTO.data);
						break;
					case "STALE":
						alert(dojo.string.substitute(messages.welcomeErrorExists, [responseDTO.msg]));
						self._dashboardPage._initializeContentWidget(responseDTO.data);
						break;
					case "ERROR":
						alert(responseDTO.msg);
						break;
				}
			},
			failure: function(errorObj) {
				console.error("Dashboard.js remote error: " + errorObj.message);
			}
		};
		this._dashboardPage._createContentWidget();
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.createDashboard(this._scope, this._scopeItemId, true, srh);
	}
});

})();