/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.TeamsViewlet");

dojo.require("com.ibm.team.dashboard.viewlets.web.client.internal.TeamsClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dijit.Tree");
dojo.require("dojo.data.ItemFileReadStore");
dojo.require("dojo.string");

dojo.require("dojo.i18n");
dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "TeamsMessages");

(function() {

var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var TeamsClient = com.ibm.team.dashboard.viewlets.web.client.internal.TeamsClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;
var messages;

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.TeamsViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/TeamsViewlet.html"),
	
	constructor: function(args) {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "TeamsMessages");
	},

	init: function() {
		this._updateTitle();
	},
	
	refresh: function() {
		// handle Auto(none) scope when no team areas
		if (!this.getScopeItem()) {
			this._content.innerHTML = "";
			this._content.appendChild(this._createNoScopeMsg());
			this.getSite().setSecondaryTitle("");
			this.getSite().doneRefresh();
			return;
		}
		this.getSite().setLoading(true);
		this._hasMultipleLevels = false;
		var self = this;
		var serviceObj = {
			success: function(teams) {
				if (!teams || teams.length == 0) {
					delete self._treeData;
					delete self._treeStore;
					self.getSite().setSecondaryTitle("");
				} else {
					var pathname = window.location.pathname;
					var params = window.location.search;
					self._projectPath = pathname.substring(0,pathname.lastIndexOf('/') + 1) +  encodeURIComponent(teams[0].projectName) + (params ? params : "");
					self._checkTeams(teams);
					self.getSite().setSecondaryTitle("(" + self._count + ")");
					self._treeData = new Object();
					self._treeData.label = "name";
					self._treeData.items = teams;
					self._treeStore = new dojo.data.ItemFileReadStore({data: self._treeData});
				}
				self.update();
				self.getSite().setLoading(false);
				self.getSite().doneRefresh();
			},
			failure: function(errorObj) {
				self.getSite().doneRefresh();
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [errorObj]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		if (this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA)
			TeamsClient.getTeamsForProjectArea(this.getScopeItem().itemId, this.getScopeSubteams(), srh, this.getServiceTracker())
		else
			TeamsClient.getTeamsForTeamArea(this.getScopeItem().itemId, this.getScopeSubteams(), srh, this.getServiceTracker())
	},
	
	update: function() {
		this._content.innerHTML = "";
		if (this._teamsTree)
			this._teamsTree.destroy();
		if (this._treeStore) {
			this._teamsTree = new com.ibm.team.dashboard.viewlets.web.ui.internal.TeamsViewletTree({
				store: this._treeStore,
				labelAttr: "name",
				viewlet: this
			});
			this._content.appendChild(this._teamsTree.domNode);
			if (!this._hasMultipleLevels)
				dojo.addClass(this._teamsTree.domNode, "topLevelOnlyTree");
			if (this.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA))
				dojo.addClass(this._teamsTree.domNode, "renderLinks");
		} else {
			this._content.innerHTML = this.getScope() == DashboardConstants.SCOPE_PROJECT_AREA ? 
				messages.noTeamsForProject : messages.noTeamsForTeam;
		}
	},
	
	_checkTeams: function(teams) {
		this._count = teams.length;
		teams.sort(this._sortTeams);
		for (var i = 0; i < teams.length; i++) {
			if (teams[i].children) {
				this._hasMultipleLevels = true;
				this._checkSubTeams(teams[i].children);
			}
		}
	},
	
	_checkSubTeams: function(teams) {
		this._count += teams.length;
		teams.sort(this._sortTeams);
		for (var i = 0; i < teams.length; i++) {
			if (teams[i].children) {
				this._checkSubTeams(teams[i].children);
			}
		}
	},
	
	_createNoScopeMsg: function() {
		var msgSpan = document.createElement("span");
		msgSpan.innerHTML = messages.notScoped;
		if (this.getSite().isDashboardEditing()) {
			var editSpan = document.createElement("span");
			var nodes = I18nUtil.addMessage(editSpan, messages.notScopedEdit);
			this.connect(nodes[0], "onclick", this._handleToggleSettings);
			msgSpan.appendChild(editSpan);
		}
		return msgSpan;
	},
	
	_handleToggleSettings: function(event) {
		dojo.stopEvent(event);
		this.getSite().toggleSettings();
	},

	_sortTeams: function(a,b) {
		return a.name.toLowerCase() < b.name.toLowerCase() ? -1 : 1;
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		var scopeChanged = false;
		var updateTitle = false;
		if (newSettings.scope != oldSettings.scope || newSettings.scopeItemId != oldSettings.scopeItemId) {
			updateTitle = true;
			scopeChanged = true;
		}
		if (updateTitle || newSettings.userTitle != oldSettings.userTitle)
			this._updateTitle();
		if (scopeChanged || (newSettings.scope != DashboardConstants.SCOPE_CONTRIBUTOR && newSettings.scopeSubteams != oldSettings.scopeSubteams))
			this.getSite().refresh();
		else 
			this.update();
	},
	
	_updateTitle: function() {
		this.getSite().setTitle(this.getScopeItem() ? dojo.string.substitute(messages.titleScoped, [this.getScopeItem().name]) : messages.title);
	}
});

dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.TeamsViewletTree",
	dijit.Tree, {
	
	postCreate: function() {
		var oldState = this.viewlet.getContributorState().get("treeState");
		this._openedItemIds = {};
		if(oldState){
			if (this.viewlet.getScope() == this.viewlet.getContributorState().get("scope") &&
					this.viewlet.getScopeItem().itemId == this.viewlet.getContributorState().get("scopeItemId"))
				this._openedItemIds = oldState;
		}
		// disable persistance for inherited postCreate so dijit.Tree does not try to use a cookie
		this.persist = false;
		this.inherited("postCreate", arguments);
		this.restoreState();
		// enable persistance so _saveState is called
		this.persist = true;
	},
	
	onClick: function(item, node) {
		if (this.viewlet.getSite().canViewDashboard(DashboardConstants.SCOPE_TEAM_AREA) && this.viewlet.getSite().navigateAwayEvent(false, true))
			location.href= this.viewlet._projectPath + "#action=" + DashboardConstants.ACTION_VIEW_DASHBOARD + "&" + DashboardConstants.URL_PARAM_TEAM + "=" + item.path;
	},
	
	restoreState: function() {
		// Get the tree's top level nodes
		this._restoreNodeListState(this.getChildren());
	},
	
	_restoreNodeListState: function(treeNodeList) {
		if ((treeNodeList == null) ||
			(treeNodeList.length == 0)) {
			// Nothing to do
			return;
		}
		for (var i = 0; i < treeNodeList.length; i++) {
			var identity = this.store.getIdentity(treeNodeList[i].item);
			if(this._openedItemIds[identity]){
				this._expandNode(treeNodeList[i]);
			}
			this._restoreNodeListState(treeNodeList[i].getChildren());
		}
	},
	
	_saveState: function(){
		this.viewlet.getContributorState().set("treeState",this._openedItemIds);
		if (this.viewlet.getContributorState().get("scope") != this.viewlet.getScope())
			this.viewlet.getContributorState().set("scope",this.viewlet.getScope());
		if (this.viewlet.getContributorState().get("scopeItemId") != this.viewlet.getScopeItem().itemId)
			this.viewlet.getContributorState().set("scopeItemId",this.viewlet.getScopeItem().itemId);
	}
	
});

})();
