/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.settings.SettingsArea");

dojo.require("dojo.i18n");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dijit.form.Form");
dojo.require("dijit.layout.TabContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.PreferenceTab");
dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.AppearanceTab");
dojo.require("com.ibm.team.dashboard.web.ui.internal.settings.ScopeTab");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ResizeNotifier");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var PreferenceTab = com.ibm.team.dashboard.web.ui.internal.settings.PreferenceTab;
var AppearanceTab = com.ibm.team.dashboard.web.ui.internal.settings.AppearanceTab;
var ScopeTab = com.ibm.team.dashboard.web.ui.internal.settings.ScopeTab;
var ResizeNotifier = com.ibm.team.dashboard.web.ui.internal.util.resizeNotifier();

/*
 * The widget containing the viewlet-specific preference fields.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.settings.SettingsArea",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/SettingsArea.html"),
	
	constructor: function(args) {
		this.viewlet = args.viewlet;
		this.viewletHelper = args.viewletHelper;
		this.viewletDefinition = args.viewletDefinition;
		this.viewletContainer = args.self;
	},
	
	init: function() {
		var form = new dijit.form.Form({execute:dojo.hitch(this, this._saveSettings)})		
		this._settingsArea.appendChild(form.domNode);
		
		this._createTabs();
		
		this._calculateHeight();
		var tabDiv = document.createElement("div");
		tabDiv.style.width="100%";
		form.domNode.appendChild(tabDiv);
		this.tabContainer = new dijit.layout.TabContainer({style: "height:"+this.height+"px"},tabDiv);
		
		this._initWorkingCopy();
		this._setupTabs();
		
		this._submitArea = new com.ibm.team.dashboard.web.ui.internal.settings.SubmitArea();
		this.connect(this._submitArea.cancelButton,"onmouseup",this._cancelSettings);
		form.domNode.appendChild(this._submitArea.domNode);
		
		this._windowResizeEvent = ResizeNotifier.register({context: this, callback: this._resize, browserEvents: false, force: true});
		this._windowResizeEvent2 = ResizeNotifier.register({context: this, callback: this._checkWrap, publishEvents: false});
		this._checkWrap();
	},
	settingChanged: function(settingId) {
		for (var i = 0; i < this.tabs.length; i++)
			this.tabs[i].settingChanged(settingId);
	},
	_createTabs: function() {
		var potentialTabs = new Array();
		this.tabs = new Array();
		potentialTabs.push(new PreferenceTab({settingsArea: this},document.createElement("div")));
		potentialTabs.push(new AppearanceTab({settingsArea: this},document.createElement("div")));
		potentialTabs.push(new ScopeTab({settingsArea: this},document.createElement("div")));
		for (var i = 0; i < potentialTabs.length; i++) {
			if (potentialTabs[i].isRequired())
				this.tabs.push(potentialTabs[i]);
			else 
				potentialTabs[i].destroy();
		}
	},
	_resize: function() {
		this.tabContainer.resize();
		this._checkWrap();
	},
	_checkWrap: function() {
		var node = this.tabContainer.tablist.domNode;
		if (node && node.clientHeight > 25) {
			var tc = this.tabContainer.domNode;
			this.tabContainer.resize({h: this.height + (node.clientHeight - 24)});
			this.twoTabRows = true;
		} else if (this.twoTabRows) {
			this.tabContainer.resize({h: this.height});
			this.twoTabRows = false;
		}
	},
	_cancelSettings: function() {
		for (var i = 0; i < this.tabs.length; i++)
			this.tabs[i].cancelSettings();
		this.viewletContainer.toggleSettings();
	},
	_calculateHeight: function() {
		var numFields = 0;
		for (var i=0; i<this.tabs.length; i++) {
			var fields = this.tabs[i].getNumberOfFields();
			if (fields > numFields)
				numFields = fields;
		};
		this.height = 39 + 28 * numFields;
	},
	_setupTabs: function() {
		for (var i = 0; i < this.tabs.length; i++)
			this.tabs[i].setup();
		this.tabContainer.startup();
	},
	_initWorkingCopy: function() {
		this.workingCopy = new Object();
		for (var i = 0; i < this.tabs.length; i++)
			this.tabs[i].initWorkingCopy();
		this.oldSettings = dojo.clone(this.workingCopy);
	},
	_saveSettings: function(/* object */ newSettings) {
		this.viewletHelper.getDashboardItem().startTransaction();
		for (var i = 0; i < this.tabs.length; i++)
			this.tabs[i].saveSettings();
		this.viewletHelper.getDashboardItem().endTransaction();
		
		
		this.viewletContainer.toggleSettings();
		
		// inform the viewlet that settings have changed
		// it is responsible for setting title, refreshing, updating, etc.
		this.viewlet.settingsChanged(this.oldSettings, this.workingCopy);
	},
	destroy: function() {
		if (this._submitArea)
			this._submitArea.destroy();
		if (this.tabs) {
			for (var i = 0; i < this.tabs.length; i++)
				this.tabs[i].destroy();
		}
		ResizeNotifier.unregister(this._windowResizeEvent);
		ResizeNotifier.unregister(this._windowResizeEvent2);
		this.tabContainer.destroy();
		this.inherited(arguments);
	}
});

dojo.declare("com.ibm.team.dashboard.web.ui.internal.settings.SubmitArea",
	[dijit._Widget, dijit._Templated], {
	
	constructor: function() {
		this._messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	
	templateString: "<div class=\"submit-area\">" +
			"<div class=\"required-field\">* ${_messages.settingsReq}</div>" +
			"<input class=\"save-button\" type=\"submit\" value=\"${_messages.save}\"/>" +
			"<input dojoAttachPoint=\"cancelButton\" class=\"cancel-button\" type=\"button\" value=\"${_messages.cancel}\"/>" +
			"</div>"
		
});

})();
