/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.FormatUtil");

(function() {
	
var DELIMITERS				= [".", "-", "_", "/", "?", "#"];
if (dojo.isIE && dojo.isIE < 7) {
	var MAX_LEN					= 40;
} else {
	var MAX_LEN					= 52;
}

com.ibm.team.dashboard.web.ui.internal.util.FormatUtil = {
	
	insertWordBreaks: function(paragraph, minLength, /* boolean */ escape) {
		// break the overall paragraph into strings based on spaces
		var words= paragraph.split(" ");
		var needsBreakingUp= false;
		// two pass algorithm so that we don't have to do a lot of string
		// copying for the 99% case
		for (var i=0; i<words.length; i++) {
			if (words[i].length > minLength) {
				needsBreakingUp= true;
				break;
			}
		}
		if (!needsBreakingUp) {
			return (escape ? this.escapeCommonCharacters(paragraph) : paragraph);
		}
		// if we're still running we have some long words, so now we'll do
		// some string manipulation
		for (i=0; i<words.length; i++) {
			// if this word is too long, break it up
			if ( words[i].length >= minLength ) {
				words[i] = this._breakupLongWord(words[i], minLength, escape);
			} else if (escape) {
				words[i] = this.escapeCommonCharacters(words[i]);
			}
		}
		return words.join(" ");
	},
	
	_breakupLongWord: function(word, minLength, /* boolean */ escape) {
		if(!minLength) {
			minLength =MIN_BREAKUP_LEN;
		}
		var remainingWord= word;
		var brokenupWord= "";
		while (remainingWord.length > minLength) {
			var splitPoint= minLength + 1;
			for (var i=0; i<DELIMITERS.length; i++) {
				var delim= DELIMITERS[i];
				var end= (remainingWord.length > MAX_LEN ? MAX_LEN : remainingWord.length) + 1;
				var candidateSplitSubstr= remainingWord.substring(minLength + 1, end);
				var delimIndex= candidateSplitSubstr.indexOf(delim);
				if (delimIndex != -1) {
					splitPoint=  minLength + delimIndex + 2;	
					break;
				}
			}
			brokenupWord += (escape ? this.escapeCommonCharacters(remainingWord.substring(0, splitPoint)) : remainingWord.substring(0, splitPoint)) + "<wbr/>";
			remainingWord = remainingWord.substring(splitPoint, remainingWord.length);
		}
		brokenupWord += (escape ? this.escapeCommonCharacters(remainingWord) : remainingWord);
		return brokenupWord;
	},
	
	escapeCommonCharacters: function (string) {
		if (!string)
			return string;
		var resultArr = [];
		for (var i = 0; i < string.length; i++) {
			var char = string.charAt(i);
			switch(char) {
			case '<':
				resultArr.push("&lt;");
				break;
			case '>':
				resultArr.push("&gt;");
				break;
			case '"':
				resultArr.push("&quot;");
				break;
			case '&':
				resultArr.push("&amp;");
				break;
			default:
				resultArr.push(char);
				break;
			}
		}
		return resultArr.join("");
	}
}
	
})();
