/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.common.TableTreeItem"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$

(function() {

var Assert				= com.ibm.team.apt.web.ui.internal.common.Assert;

var TREENODEID			= "tnid"; //$NON-NLS-1$
var TREENODELEVEL		= "tnlevel"; //$NON-NLS-1$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.common.TableTreeItem", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------
	
	_parentItem: null,
	_itemClass: null,
	_nodeId: null,
	
	_row: null,
	
	_isExpansionInProgress: false,
	
	constructor: function() {
		this._nextChildId= 0;
		this._lazyChildCreator= null;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	expand: function() {
		this._setExpansionState(true);
	},

	collapse: function() {
		this._setExpansionState(false);
	},
	
	isExpanded: function() {
		// a root node is always expanded
		return this._parentItem == null || dojo.hasClass(this._row, "expanded"); //$NON-NLS-1$
	},
	
	toggleExpansionState: function() {
		this._setExpansionState(!this.isExpanded());
	},

	setExpansionState: function(doExpand) {
		this._setExpansionState(doExpand);
	},
	
	setLazyChildCreator: function(listener) {
		this._lazyChildCreator= listener;
		if (dojo.hasClass(this._row, "nochildren")) { //$NON-NLS-1$
			dojo.removeClass(this._row, "nochildren"); //$NON-NLS-1$
			dojo.addClass(this._row, "collapsed"); //$NON-NLS-1$
		}
	},

	// ---- subclass API ---------------------------------------------------------------------------------------------------
	
	_getParentItem: function() {
		return this._parentItem;
	},
	
	_applyTreeLayout: function(handleExpansion) {
		var firstCell= this._getTreeLayoutElement();

		dojo.style(firstCell, "paddingLeft",  (4 + (this._treeLevel - 1) * 16) + "px"); //$NON-NLS-1$ //$NON-NLS-2$
		dojo.addClass(this._row, "nochildren"); //$NON-NLS-1$
		
		dojo.toggleClass(this._row, "hidden", this._parentItem != null && !this._parentItem.isExpanded()); //$NON-NLS-1$

		var twisty= document.createElement("span"); //$NON-NLS-1$
		dojo.addClass(twisty, "twisty"); //$NON-NLS-1$
		firstCell.insertBefore(twisty, firstCell.firstChild);

		if (handleExpansion)
			this.connect(twisty, "onclick", "toggleExpansionState"); //$NON-NLS-1$ //$NON-NLS-2$
	},

	_getRowContainer: function() {
		return this._row;
	},

	_getTableContainer: function() {
		return this._row.parentNode;
	},
	
	_addChild: function(childTreeItem) {
		if (dojo.hasClass(this._row, "nochildren")) { //$NON-NLS-1$
			dojo.removeClass(this._row, "nochildren"); //$NON-NLS-1$
			dojo.addClass(this._row, "collapsed"); //$NON-NLS-1$
		}
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templateString: "<tr dojoAttachPoint=\"_row\"></tr>", //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		dojo.addClass(this._row, this._itemClass);
		this._row.setAttribute(TREENODEID, this._nodeId);
		this._row.setAttribute(TREENODELEVEL, this._treeLevel);
		
		if (this._parentItem != null) {
			this._parentItem._addChild(this);
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_getTreeLayoutElement: function() {
		var queryResult= dojo.query("td:first-child", this._row); //$NON-NLS-1$
		return queryResult.length > 0 ? queryResult[0] : null;
	},

	_setExpansionState: function(doExpand) {
		if (!this._isExpansionInProgress && !dojo.hasClass(this._row, "nochildren")) { //$NON-NLS-1$
			this._isExpansionInProgress= true;
			
			if (doExpand && this._lazyChildCreator != null) {
				this._lazyChildCreator();
				this._lazyChildCreator= null;
			}
			
			dojo.toggleClass(this._row, "expanded", doExpand); //$NON-NLS-1$			
			dojo.toggleClass(this._row, "collapsed", !doExpand); //$NON-NLS-1$

			var nodeToggler= dojo.hitch(this, function(nodesToToggle, yieldStartIndex) {
				var yieldEndIndex= doExpand ? Math.min(nodesToToggle.length, yieldStartIndex + 20) : nodesToToggle.length;
				
				for (var i= yieldStartIndex; i < yieldEndIndex; i++) {
					var node= nodesToToggle[i];
					if (doExpand) {
						dojo.removeClass(node, "hidden"); //$NON-NLS-1$
					} else {
						dojo.addClass(node, "hidden"); //$NON-NLS-1$
					}

					// define the state of all child tree nodes as collapsed
					dojo.removeClass(node, "expanded"); //$NON-NLS-1$
					dojo.addClass(node, "collapsed"); //$NON-NLS-1$
				}

				if (nodesToToggle.length > yieldEndIndex) {
					window.setTimeout(function() { nodeToggler(nodesToToggle, yieldEndIndex) }, 0);
				} else {
					this._isExpansionInProgress= false;
				}
			});
			
			nodeToggler(dojo.query(this._getTreeLevelSelector(doExpand), this._getTableContainer()), 0);
		}
	},

	_getTreeLevelSelector: function(directChildrenOnly) {
		var result= "tr[tnid^=\"" + this._nodeId + "-\"]"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		if (directChildrenOnly) {
			result+= "[tnlevel=\"" + (this._treeLevel + 1) + "\"]" //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
		}
		return result + ""; //$NON-NLS-1$
	},

	_generateChildNodeId: function() {
		var result= ""; //$NON-NLS-1$
		if (this._nodeId != null) {
			result+= this._nodeId + "-"; //$NON-NLS-1$
		}
		
		return result + (this._nextChildId++);
	},

	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.web.ui.internal.view.common.TableTreeItem.constructorArgs= function(parentItem, itemClass) {
	var nodeId= null;
	var treeLevel= -1;
	
	if (parentItem != null) {
		nodeId= parentItem._generateChildNodeId();
		
		// we can derive the tree level from the nodeId
		treeLevel= 0;

		var anchestor= parentItem;
		while (anchestor != null) {
			treeLevel++;
			anchestor= anchestor._getParentItem();
		}
	}

	return {
		_parentItem: parentItem,
		_itemClass: itemClass,
		_nodeId: nodeId,
		_treeLevel: treeLevel
	};
};

})();
