/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.common.TreeNode"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.common.TableTreeItem"); //$NON-NLS-1$

(function() {

var TableTreeItem		= com.ibm.team.apt.web.ui.internal.view.common.TableTreeItem;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.common.TreeNode", null, { //$NON-NLS-1$

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	constructor: function(nodeId, nodeType, nodeValue, widgetFactory, parent) {
		this._nodeValue= nodeValue;
		this._widgetFactory= widgetFactory;
		this._parent= parent;
		this._nodeId= nodeId;
		this._nodeType= nodeType;
		this._widget= null;
		this._children= null;
		this._tags= {};
		this._filtered= false;
		
		if (this._parent != null) {
			this._parent._addChild(this);
		}
		
		if (this.isRootNode()) {
			// root nodes must produce invisible dummy TableTreeItems
			this._widgetFactory= function() {
				return new TableTreeItem(TableTreeItem.constructorArgs(null, null));
			}
		}
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	getNodeId: function() {
		return this._nodeId;
	},
	
	getNodeType: function() {
		return this._nodeType;
	},
	
	getNodeValue: function() {
		return this._nodeValue;
	},
	
	isRootNode: function() {
		return this._parent == null;
	},
	
	getParent: function() {
		return this._parent;
	},
	
	hasChildren: function() {
		return this._children != null && this._children.length > 0;
	},

	getChildren: function() {
		return this._children || [];
	},

	getChild: function(nodeId) {
		if (this.hasChildren()) {
			for (var i= 0; i < this._children.length; i++) {
				if (this._children[i].getNodeId() == nodeId) {
					return this._children[i];
				}
			}
		}
		return null;
	},
	
	getWidget: function(parentTreeItem) {
		if (this._widget == null) {
			this._widget= this._widgetFactory(this, parentTreeItem);
		}
		return this._widget;
	},

	getCreatedWidget: function() {
		return this._widget;
	},

	setTag: function(tag) {
		this._tags[tag]= true;
	},

	clearTag: function(tag) {
		this._tags[tag]= false;
	},

	hasTag: function(tag) {
		return this._tags[tag] === true;
	},

	isFiltered: function() {
		return this._filtered;
	},

	setFiltered: function(filtered) {
		this._filtered= filtered;
	},

	accept: function(visitor) {
		if (this.isRootNode() || visitor(this)) {
			var children= this.getChildren();
			for (var i= 0; i < children.length; i++) {
				children[i].accept(visitor);
			}
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	_addChild: function(child) {
		if (this._children == null) {
			this._children= [];
		}
		
		this._children.push(child);
	},
	
	__sentinel: null // terminates this class definition
});

})();
