/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.ListWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.page.EventTopics");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");

(function(){

var EventTopics= com.ibm.team.workitem.web.ui.internal.page.EventTopics;
var ImageClient= com.ibm.team.workitem.web.client.internal.ImageClient;
var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient= com.ibm.team.workitem.web.client.internal.WorkItemClient;

dojo.declare("com.ibm.team.workitem.web.ui.internal.view.queryeditor.widgets.ListWidget", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/queryeditor/widgets/templates/ListWidget.html"),
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {

		this.presentation= args.presentation;
		this._selection= args.selection || [];
		
		this._idLookup= new Object();
		this._selectMap= new Object();
		
		this._hasSelection= false;
	},
	
	postCreate: function() {
		
		this._select.size= 6;
		this._select.multiple= "multiple";
		
		var attributeId= this.presentation.attribute.id;
		var srh= new ServiceResponseHandler(this, "_initValues", "_handleError");
		WorkItemClient.getAllValues(srh, { newWorkItem: true, ids: [ attributeId ], ignoreDependencies: true });
	},
	
	/* String[] */ getValues: function() {
		
		if (!this._hasSelection)
			return this._getInitialSelectionValues();
		
		var selection= new Array();
		var select= this._select;
		for (var i= 0; i < select.length; i++) {
			if (select[i].selected) {
				selection.push(select[i].value);
			}
		}
		return selection;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_getInitialSelectionValues: function() {
		var values= this._selection;
		var selection= new Array();
		for (var i= 0; i < values.length; i++) {
			selection.push(values[i].id);
		}
		return selection;
	},
	
	_onSelectionChange: function() {
		dojo.publish(EventTopics.QUERY_EDITOR_CHANGED, []);
	},
	
	_initValues: function(/* UIItemSetDTO */ valueSet) {
		if (!valueSet || valueSet.length == 0)
			return;
		this._select.length= 0;
		this._addValues(valueSet[0].uiItems);
		this._selectValues(this._selection);
	},
	
	_handleError: function(e) {
		if (this.errorCallback)
			this.errorCallback("Loading values failed", e);
	},
	
	_addValues: function(/* UIItemDTO[] */ values, /* boolean */ select) {
		if (!values || values.length == 0)
			return;
		var index= this._select.length;
		for (var i= 0; i < values.length; i++) {
			var option= this._createOption(values[i]);
			this._select[index++]= option;
			this._selectMap[values[i].id]= option;
			option.selected= select ? true : false;
		}
	},
	
	_selectValues: function(/* UIItemDTO[] */ selection) {
		this._hasSelection= true;
		var missing= new Array();
		for (var i= 0; i < selection.length; i++) {
			var item= selection[i];
			var option= this._selectMap[item.id];
			if (option) {
				option.selected= true;
			} else {
				if (item.arguments) // hack for category
					item.label= item.arguments;
				missing.push(item);
			}
		}
		this._addValues(missing, true);
	},
	
	_createOption: function(/* UIItemDTO */ item) {
		var option= new Option(item.label, item.id, false, false);
		var indent= this._getIndent(item);
		if (indent > 0) {
			option.setAttribute("style", "padding-left: " + indent * 5 + "px");
		}
		if (item.iconUrl) {
			var url= ImageClient.getImageUrl(item.iconUrl);
			option.setAttribute("style", "background-image: url('" + url + "'); padding-left: 20px");
		}
		return option;
	},
	
	_getIndent: function(/* UIItemDTO */ item) {
		var label= item.label;
		var indent= 0;
		while (indent < label.length && label[indent] == ' ') 
			indent++;
		return indent;
	}
	
});
	
})();
