/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.widgets.WorkLoadBar"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.ui.internal.HoverPopup"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.WorkLoadLabelProvider"); //$NON-NLS-1$

(function() {

var PlanningUtil		= com.ibm.team.apt.web.client.util;

var replaceChildren			= dojox.data.dom.replaceChildren;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.widgets.WorkLoadBar", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_workLoadInfo: null,
	_labelProvider: null,
	_width: null,
	_height: null,
	
	_workLoadBar: null,
	_barContainer: null,
	_noWorkEstimatedLabel: null,

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getLabelProvider: function() {
		return this._labelProvider;
	},
	
	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/widgets/templates/WorkLoadBar.html"), //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		dojo.style(this._workLoadBar, "width", this.width); //$NON-NLS-1$
		dojo.style(this._workLoadBar, "font-size", this.height); //$NON-NLS-1$
		
		var message= this._labelProvider.getMessage();

		if (message != null) {
			dojo.style(this._barContainer, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			dojox.data.dom.replaceChildren(this._noWorkEstimatedLabel, document.createTextNode(message));
		} else {
			dojo.style(this._noWorkEstimatedLabel, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
			
			var sizeBooked = 0;
			var remainderClass;
			
			var workHoursLeft= this._workLoadInfo.getWorkHoursLeft();
			var realTimeLeft= this._workLoadInfo.getRealTimeLeft();
						
			if (this._workLoadInfo.isRealTimeSpecified()) {
				if (workHoursLeft <= realTimeLeft) {
					sizeBooked = realTimeLeft > 0 ? Math.round((100 * workHoursLeft / realTimeLeft)) : 0;
					remainderClass = "available"; //$NON-NLS-1$
				} else {
					sizeBooked = Math.round(100 * realTimeLeft / workHoursLeft);
					remainderClass = "overbooked"; //$NON-NLS-1$
				}

			} else {
				sizeBooked= Math.round((100 * workTimeLeft / this._workLoadInfo.getWorkHours()));
				remainderClass = "available"; //$NON-NLS-1$
			}

			var sizeRemainder = 100 - sizeBooked;

			this._applyStyle(".barContainer .notEstimated", "height", 100 - this._workLoadInfo.getQualityOfPlanning()); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .estimated", "height", this._workLoadInfo.getQualityOfPlanning()); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .booked", "width", sizeBooked); //$NON-NLS-1$ //$NON-NLS-2$
			this._applyStyle(".barContainer .remainder", "width", sizeRemainder); //$NON-NLS-1$ //$NON-NLS-2$ 
			dojo.query(".barContainer .remainder", this._workLoadBar).addClass(remainderClass); //$NON-NLS-1$
		}

		var tooltipContentSpan= document.createElement("span"); //$NON-NLS-1$
		tooltipContentSpan.innerHTML= this._labelProvider.getTooltipContent();
		
		new net.jazz.ajax.ui.internal.HoverPopup({
			connectNode: dojo.query("div", this._workLoadBar) //$NON-NLS-1$
		}, tooltipContentSpan);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_applyStyle: function(query, style, value) {
		dojo.query(query, this._workLoadBar).style(style, value + "%"); //$NON-NLS-1$
	},

	__sentinel: null // terminates this class definition
});

/**
 * This helper method creates the arguments object for the widget's constructor.
 * @return {Object} a widget private object to be passed in as a constructor argument
 */
com.ibm.team.apt.web.ui.internal.view.widgets.WorkLoadBar.constructorArgs= function(workLoadInfo, width, height) {
	return {
		_workLoadInfo: workLoadInfo,
		_labelProvider: new com.ibm.team.apt.web.ui.internal.common.WorkLoadLabelProvider(workLoadInfo),
		width: width || "100%", //$NON-NLS-1$
		height: height || "1em" //$NON-NLS-1$
	};
};

})();
